#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"
#include "line.h"
#include "label.h"

#define LABEL_MAXNUM 1024

static struct label labels[LABEL_MAXNUM];

static label_t head = NULL;

static label_t label_alloc(void)
{
  label_t label;
  int i;

  for (i = 0; i < LABEL_MAXNUM; i++) {
    label = &labels[i];
    if (label->name[0] == '\0')
      return label;
  }

  return NULL;
}

static int label_free(label_t label)
{
  label->name[0] = '\0';
  label->next = NULL;
  return 0;
}

int label_init(void)
{
  head = NULL;
  memset(labels, 0, sizeof(labels));
  return 0;
}

static int labelcmp(const char *s0, const char *s1)
{
  while (1) {
    if (((*s0 == '\0') || (*s0 == ':')) &&
	((*s1 == '\0') || (*s1 == ':')))
      break;
    if (*s0 != *s1)
      return *s0 - *s1;
    s0++;
    s1++;
  }
  return 0;
}

int label_get(const char *name, label_t *labelp)
{
  label_t label;

  for (label = head; label; label = label->next) {
    if (!labelcmp(label->name, name)) {
      *labelp = label;
      return 0;
    }
  }

  return NLL_ERRCODE_LABEL_NOT_FOUND;
}

int label_set(const char *name, line_t line, int column)
{
  label_t label;
  int i, len = 0;

  for (i = 0; name[i] && (name[i] != ':'); i++)
    len++;
  if (len > LABEL_NAMELEN)
    return NLL_ERRCODE_LABEL_TOO_LONG;

  for (label = head; label; label = label->next) {
    if (!labelcmp(label->name, name))
      break;
  }

  if (!label) {
    label = label_alloc();
    if (!label)
      return NLL_ERRCODE_LABEL_BUFFER_OVER;
    label->next = head;
    head = label;
  }

  memcpy(label->name, name, len);
  label->name[i] = '\0';

  label->line   = line;
  label->column = column;

  return 0;
}

int label_del(line_t line)
{
  label_t *p, next;

  for (p = &head; *p;) {
    if ((*p)->line == line) {
      next = (*p)->next;
      label_free(*p);
      *p = next;
    } else {
      p = &((*p)->next);
    }
  }

  return 0;
}

int label_print(void)
{
  label_t label;

  for (label = head; label; label = label->next) {
    printf("%s:\n", label->name);
  }

  return 0;
}
