#include <stdio.h>
#include <stdlib.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"

void nll_error_print(const char *filename, int line,
		     int errcode, const char *param)
{
  if (!param)
    param = "(?)";

  fprintf(stderr, "^C\n");

  switch (errcode) {
  case NLL_ERRCODE_FILE_NOT_FOUND:
    fprintf(stderr, "File not found: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_BUFFER_OVER:
    fprintf(stderr, "Line buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_TOO_LONG:
    fprintf(stderr, "Too long line: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_NOT_FOUND:
    fprintf(stderr, "Line not found: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_INVALID_NAME:
    fprintf(stderr, "Invalid variable name: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_STACK_OVER:
    fprintf(stderr, "Value stack overflow: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_LESS_PARAMETER:
    fprintf(stderr, "Less parameter in operand: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_DIV_ZERO:
    fprintf(stderr, "Division by zero: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_INVALID_OPERATOR:
    fprintf(stderr, "Invalid operator: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_INVALID_VALUE:
    fprintf(stderr, "Invalid value: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_UNSUPPORTED_OPERATOR:
    fprintf(stderr, "Unsupported operator: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_NO_CONTINUE_POINT:
    fprintf(stderr, "No continue point: %s\n", param);
    break;
  case NLL_ERRCODE_WORD_TOO_LONG:
    fprintf(stderr, "Word buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_BUFFER_OVER:
    fprintf(stderr, "Label buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_TOO_LONG:
    fprintf(stderr, "Too long label: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_NOT_FOUND:
    fprintf(stderr, "Label not found: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_STACK_OVER:
    fprintf(stderr, "Position stack overflow: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_LESS_PARAMETER:
    fprintf(stderr, "Less position to return: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_DIFFERENT_TYPE:
    fprintf(stderr, "Different position type: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_DIFFERENT_VARIABLE:
    fprintf(stderr, "Different position variable: %s\n", param);
    break;

  case NLL_ERRCODE_UNKNOWN:
  default:
    fprintf(stderr, "Unknown error: (%s:%d): %s\n", filename, line, param);
    break;
  }
}

void nll_error_exit(const char *filename, int line,
		    int errcode, const char *param)
{
  nll_error_print(filename, line, errcode, param);
  exit(1);
}

static int finished = 0;

void nll_finished_clear(void)
{
  finished = 0;
}

void nll_finish(void)
{
  finished = 1;
}

int nll_is_finished(void)
{
  return finished;
}
