#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"
#include "line.h"
#include "label.h"
#include "position.h"
#include "nll.h"

#define LINE_MAXNUM 1024

static struct line lines[LINE_MAXNUM];

static line_t head = NULL;

line_t line_alloc(void)
{
  line_t line;
  int i;

  for (i = 0; i < LINE_MAXNUM; i++) {
    line = &lines[i];
    if (line->buffer[0] == '\0') {
      line->next = NULL;
      return line;
    }
  }

  return NULL;
}

int line_free(line_t line)
{
  int r;
  position_t position;

  if ((r = nll_clear_line(line)) < 0) return r;
  if ((r = label_del(line)) < 0) return r;
  if ((r = position_get(line, -1, &position)) < 0) return r;

  if (position) {
    position_init();
    if ((r = nll_clear_line(NULL)) < 0) return r;
  }

  line->buffer[0] = '\0';
  line->next = NULL;

  return 0;
}

int line_init(void)
{
  head = NULL;
  memset(lines, 0, sizeof(lines));
  return 0;
}

int line_clear(void)
{
  line_t line;
  int r;

  while (head) {
    line = head;
    head = head->next;
    if ((r = line_free(line)) < 0) {
      head = NULL;
      return r;
    }
  }

  return 0;
}

line_t line_head(void)
{
  return head;
}

int line_is_exist(line_t line)
{
  line_t ln;

  for (ln = head; ln; ln = ln->next) {
    if (ln == line)
      return 1;
  }

  return 0;
}

int line_get(int number, line_t *linep)
{
  line_t line;
  int n = 0;

  for (line = head; line; line = line->next) {
    n++;
    if (number == n) {
      *linep = line;
      return 0;
    }
  }

  *linep = NULL;
  return 0;
}

int line_set(int number, int number2, char *buffer, line_t *linep)
{
  line_t line = NULL, next, *p = NULL, *p2 = NULL;
  int r, len = 0, n;

  if (buffer)
    len = strlen(buffer);

  if (len > LINE_MAXLEN)
    return NLL_ERRCODE_LINE_TOO_LONG;

  if (number2 >= 0) {
    n = 0;
    for (p2 = &head; *p2; p2 = &((*p2)->next)) {
      n++;
      if ((number2 > 0) && (number2 <= n))
	break;
    }
  }

  if (number >= 0) {
    n = 0;
    for (p = &head; *p; p = &((*p)->next)) {
      n++;
      if ((number > 0) && (number <= n))
	break;
      if ((number == 0) && ((len == 0) || p2) && ((*p)->next == NULL))
	break; /* delete or move last line */
    }
  }

  if (!p) { /* direct mode */
    if (len == 0) { /* empty line */
      if (linep)
	*linep = NULL;
      return 0;
    }
  } else { /* program mode */
    if ((len == 0) && !p2) { /* delete line */
      line = *p;
      if (*p) {
	next = (*p)->next;
	*p = next;
	r = line_free(line);
	if (r < 0) return r;
      }
      if (linep)
	*linep = NULL;
      return 0;
    }

    if (p2) { /* move line */
      line = *p;
      if (*p) {
	if (p2 == &((*p)->next))
	  p2 = p;
	next = (*p)->next;
	*p = next;
	if (len == 0) { /* no changed */
	  buffer = NULL;
	} else { /* changed */
	  r = line_free(line);
	  if (r < 0) return r;
	  line = NULL;
	}
      }
      if (!line && (len == 0)) {
	if (linep)
	  *linep = NULL;
	return 0;
      }
      p = p2;
    }
  }

  if (!line) {
    line = line_alloc();
    if (!line)
      return NLL_ERRCODE_LINE_BUFFER_OVER;
  }

  if (buffer) {
    memcpy(line->buffer, buffer, len);
    line->buffer[len] = '\0';
  }

  if (p) { /* program mode */
    line->next = *p;
    *p = line;
  }

  if (linep)
    *linep = line;

  return 0;
}

int line_print(int start_number, int end_number)
{
  line_t line;
  int number = 0;

  for (line = head; line; line = line->next) {
    number++;
    if ((start_number >= 0) && (number < start_number))
      continue;
    if ((end_number >= 0) && (number > end_number))
      continue;
    printf("%d%s\n", number, line->buffer);
  }

  return 0;
}
