#include <stdlib.h>
#include <string.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"
#include "line.h"
#include "position.h"

#define POSITION_MAXNUM 32

static struct position positions[POSITION_MAXNUM];
static int position_num = 0;

int position_init(void)
{
  position_num = 0;
  return 0;
}

int position_push(position_type_t type, line_t line, int column,
		  position_param_t param)
{
  position_t position;

  if (position_num >= POSITION_MAXNUM)
    return NLL_ERRCODE_POSITION_STACK_OVER;

  position = &positions[position_num];
  position->type   = type;
  position->line   = line;
  position->column = column;
  if (param)
    memcpy(&position->param, param, sizeof(*param));

  position_num++;

  return 0;
}

int position_pop(position_type_t type, line_t *linep, int *columnp,
		 position_param_t param)
{
  position_t position;

  if (position_num == 0)
    return NLL_ERRCODE_POSITION_LESS_PARAMETER;

  position_num--;

  position = &positions[position_num];
  if (type != position->type)
    return NLL_ERRCODE_POSITION_DIFFERENT_TYPE;
  *linep   = position->line;
  *columnp = position->column;
  if (param)
    memcpy(param, &position->param, sizeof(*param));

  return 0;
}

int position_get(line_t line, int column, position_t *positionp)
{
  position_t position;
  int i;

  for (i = 0; i < position_num; i++) {
    position = &positions[i];
    if ((position->line == line) &&
	((column < 0) || (position->column == column))) {
      *positionp = position;
      return 0;
    }
  }

  *positionp = NULL;
  return 0;
}
