#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"
#include "variable.h"

#define ALPHA_NUM ('Z' - 'A' + 1)
#define DIGIT_NUM ('9' - '0' + 1)
#define ALNUM_NUM (ALPHA_NUM + DIGIT_NUM)

#define VARIABLES_NUM (ALPHA_NUM + (ALPHA_NUM * ALNUM_NUM))

struct variable {
  int integer;
};

struct variable variables[VARIABLES_NUM];

int variable_init(void)
{
  memset(variables, 0, sizeof(variables));
  return 0;
}

static int alnum_index(char c)
{
  if (isalpha(c))
    return toupper(c) - 'A';
  if (isdigit(c))
    return c - '0' + ALPHA_NUM;
  return 0;
}

static int variable_index(const char *name)
{
  int index = NLL_ERRCODE_VARIABLE_INVALID_NAME;

  if (isalpha(name[0])) {
    index = alnum_index(name[0]);
    if (name[1] && isalnum(name[1]))
      index = ALPHA_NUM + (index * ALNUM_NUM) + alnum_index(name[1]);
  }

  return index;
}

int variable_get_integer(const char *name, int *integerp)
{
  int index;
  if ((index = variable_index(name)) < 0) return index;
  *integerp = variables[index].integer;
  return 0;
}

int variable_set_integer(const char *name, int integer)
{
  int index;
  if ((index = variable_index(name)) < 0) return index;
  variables[index].integer = integer;
  return 0;
}
